package com.ca.chorus.experiement;

import java.util.concurrent.ExecutionException;
import java.util.concurrent.atomic.AtomicBoolean;

import org.teiid.CommandContext;
import org.teiid.CommandListener;
import org.teiid.language.QueryExpression;
import org.teiid.metadata.RuntimeMetadata;
import org.teiid.translator.BaseDelegatingExecutionFactory;
import org.teiid.translator.ExecutionContext;
import org.teiid.translator.ResultSetExecution;
import org.teiid.translator.TranslatorException;

import com.google.common.cache.CacheBuilder;
import com.google.common.cache.CacheLoader;
import com.google.common.cache.LoadingCache;

public class ContinuousExecutionHandlingDelegateExecutionFactory<F, C>
    extends BaseDelegatingExecutionFactory<F, C>
{
    private final MessageBus            messageBus;

    LoadingCache<String, AtomicBoolean> endOfResultReachedMapCache = CacheBuilder.newBuilder()
                                                                                 .build(new CacheLoader<String, AtomicBoolean>()
                                                                                 {

                                                                                     @Override
                                                                                     public AtomicBoolean load(final String key) throws Exception
                                                                                     {
                                                                                         System.out.println("creating atomic boolean for "
                                                                                                            + key);
                                                                                         return new AtomicBoolean(false);
                                                                                     }
                                                                                 });

    public ContinuousExecutionHandlingDelegateExecutionFactory(final MessageBus messageBus)
    {
        this.messageBus = messageBus;
    }

    @Override
    public ResultSetExecution createResultSetExecution(final QueryExpression command,
                                                       final ExecutionContext executionContext,
                                                       final RuntimeMetadata metadata,
                                                       final C connection) throws TranslatorException
    {
        final ResultSetExecution resultSetExecution = super.createResultSetExecution(command,
                                                                                     executionContext,
                                                                                     metadata,
                                                                                     connection);
        try
        {
            final String key = executionContext.getRequestId()
                               + "-"
                               + executionContext.getPartIdentifier();
            final ContinuousExecutionHandlingDelegateExecution continuousExecutionHandlingDelegateExecution = new ContinuousExecutionHandlingDelegateExecution(this.messageBus,
                                                                                                                                                               executionContext,
                                                                                                                                                               this.endOfResultReachedMapCache.get(key),
                                                                                                                                                               resultSetExecution);
            executionContext.getCommandContext()
                            .addListener(new CommandListener()
                            {

                                public void commandClosed(final CommandContext context)
                                {
                                    ContinuousExecutionHandlingDelegateExecutionFactory.this.endOfResultReachedMapCache.invalidate(key);
                                }
                            });
            return continuousExecutionHandlingDelegateExecution;
        } catch (final ExecutionException e)
        {
            throw new TranslatorException(e);
        }
    }
}
