import java.io.PrintWriter;
import java.lang.reflect.Method;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.SQLFeatureNotSupportedException;
import java.util.Map;
import java.util.Map.Entry;
import java.util.logging.Logger;

import javax.naming.NamingException;
import javax.naming.Reference;
import javax.resource.ResourceException;
import javax.resource.cci.ConnectionFactory;
import javax.resource.cci.ConnectionSpec;
import javax.resource.cci.RecordFactory;
import javax.resource.cci.ResourceAdapterMetaData;
import javax.sql.DataSource;
import javax.transaction.HeuristicMixedException;
import javax.transaction.HeuristicRollbackException;
import javax.transaction.InvalidTransactionException;
import javax.transaction.NotSupportedException;
import javax.transaction.RollbackException;
import javax.transaction.Synchronization;
import javax.transaction.SystemException;
import javax.transaction.Transaction;
import javax.transaction.TransactionManager;
import javax.transaction.xa.XAResource;

import org.junit.Before;
import org.teiid.adminapi.Model.Type;
import org.teiid.adminapi.impl.ModelMetaData;
import org.teiid.jdbc.TeiidDriver;
import org.teiid.resource.adapter.file.FileConnectionImpl;
import org.teiid.runtime.EmbeddedConfiguration;
import org.teiid.runtime.EmbeddedServer;
import org.teiid.runtime.EmbeddedServer.ConnectionFactoryProvider;
import org.teiid.translator.ExecutionFactory;
import org.teiid.translator.FileConnection;

import com.google.common.base.Function;
import com.google.common.collect.Iterables;

public abstract class ATeiidTest
{

    static public class MockTransaction
        implements
            Transaction
    {

        public static final Transaction NULL = new MockTransaction();

        public void commit() throws RollbackException,
                            HeuristicMixedException,
                            HeuristicRollbackException,
                            SecurityException,
                            IllegalStateException,
                            SystemException
        {
            // TODO Auto-generated method stub

        }

        public boolean delistResource(final XAResource xaRes, final int flag) throws IllegalStateException,
                                                                             SystemException
        {
            // TODO Auto-generated method stub
            return false;
        }

        public boolean enlistResource(final XAResource xaRes) throws RollbackException,
                                                             IllegalStateException,
                                                             SystemException
        {
            // TODO Auto-generated method stub
            return false;
        }

        public int getStatus() throws SystemException
        {
            // TODO Auto-generated method stub
            return 0;
        }

        public void registerSynchronization(final Synchronization sync) throws RollbackException,
                                                                       IllegalStateException,
                                                                       SystemException
        {
            // TODO Auto-generated method stub

        }

        public void rollback() throws IllegalStateException, SystemException
        {
            // TODO Auto-generated method stub

        }

        public void setRollbackOnly() throws IllegalStateException,
                                     SystemException
        {
            // TODO Auto-generated method stub

        }

    }

    public static final class MockTransactionManager
        implements
            TransactionManager
    {
        public Transaction suspend() throws SystemException
        {
            return MockTransaction.NULL;
        }

        public void setTransactionTimeout(final int seconds) throws SystemException
        {
            // TODO Auto-generated method stub

        }

        public void setRollbackOnly() throws IllegalStateException,
                                     SystemException
        {
            // TODO Auto-generated method stub

        }

        public void rollback() throws IllegalStateException,
                              SecurityException,
                              SystemException
        {
            // TODO Auto-generated method stub

        }

        public void resume(final Transaction tobj) throws InvalidTransactionException,
                                                  IllegalStateException,
                                                  SystemException
        {
            // TODO Auto-generated method stub

        }

        public Transaction getTransaction() throws SystemException
        {
            return null;
        }

        public int getStatus() throws SystemException
        {
            // TODO Auto-generated method stub
            return 0;
        }

        public void commit() throws RollbackException,
                            HeuristicMixedException,
                            HeuristicRollbackException,
                            SecurityException,
                            IllegalStateException,
                            SystemException
        {
            // TODO Auto-generated method stub

        }

        public void begin() throws NotSupportedException, SystemException
        {
            // TODO Auto-generated method stub

        }
    }

    public static final class FileConnectionFactory
        extends AbstractConnectionFactory<FileConnection>
    {
        /**
         * 
         */
        private static final long serialVersionUID = 1650073091980309145L;

        private final String      dirName;

        public FileConnectionFactory()
        {
            this.dirName = "src/test/resources";
        }

        @Override
        public FileConnection getConnection() throws ResourceException
        {
            return new FileConnectionImpl(this.dirName, null, false);
        }
    }

    abstract protected Map<String, Iterable<String>> getViewsDefinitions();

    static public abstract class AbstractConnectionFactory<C extends javax.resource.cci.Connection>
        implements
            ConnectionFactory
    {
        private static final long serialVersionUID = 4906165501503764939L;

        public void setReference(final Reference reference)
        {
        }

        public Reference getReference() throws NamingException
        {
            return null;
        }

        public abstract C getConnection() throws ResourceException;

        public C getConnection(final ConnectionSpec properties) throws ResourceException
        {
            return null;
        }

        public RecordFactory getRecordFactory() throws ResourceException
        {
            return null;
        }

        public ResourceAdapterMetaData getMetaData() throws ResourceException
        {
            return null;
        }
    }

    public DataSource              teiidDataSource;

    volatile protected TeiidDriver driver;

    @Before
    public void setup() throws Exception
    {
        final EmbeddedServer server = new EmbeddedServer();
        final Method addTranslatorMethod = server.getClass()
                                                 .getDeclaredMethod("addTranslator",
                                                                    new Class[] {
                                                                            String.class,
                                                                            ExecutionFactory.class });
        addTranslatorMethod.setAccessible(true);
        final EmbeddedConfiguration embeddedConfiguration = new EmbeddedConfiguration();
        embeddedConfiguration.setTransactionManager(new MockTransactionManager());
        server.start(embeddedConfiguration);

        for (final Entry<String, ExecutionFactory<?, ?>> entries : getExecutionFactories().entrySet())
        {
            addTranslatorMethod.invoke(server,
                                       entries.getKey(),
                                       entries.getValue());
        }
        for (final Map.Entry<String, ConnectionFactoryProvider<?>> entry : getConnectionFactoryProviders().entrySet())
        {
            final String sourceName = entry.getKey();
            final ConnectionFactoryProvider<?> connectionFactoryProvider = entry.getValue();
            server.addConnectionFactoryProvider(sourceName,
                                                connectionFactoryProvider);
        }
        server.deployVDB("test",
                         Iterables.toArray(Iterables.concat(getModelMetadatas(),
                                                            Iterables.transform(getViewsDefinitions().entrySet(),
                                                                                new Function<Entry<String, Iterable<String>>, ModelMetaData>()
                                                                                {

                                                                                    public ModelMetaData apply(final Entry<String, Iterable<String>> input)
                                                                                    {
                                                                                        final String schemaName = input.getKey();
                                                                                        final ModelMetaData viewModel = new ModelMetaData();
                                                                                        viewModel.setModelType(Type.VIRTUAL);
                                                                                        viewModel.setSchemaSourceType("ddl");
                                                                                        viewModel.setName(schemaName);
                                                                                        final StringBuilder ddl = new StringBuilder();
                                                                                        for (final String viewDefinition : input.getValue())
                                                                                        {
                                                                                            ddl.append(viewDefinition);
                                                                                            ddl.append(";\n");
                                                                                        }
                                                                                        viewModel.setSchemaText(ddl.toString());
                                                                                        return viewModel;
                                                                                    }
                                                                                })),
                                           ModelMetaData.class));
        this.driver = server.getDriver();
        this.teiidDataSource = new DataSource()
        {

            public <T> T unwrap(final Class<T> iface) throws SQLException
            {
                throw new UnsupportedOperationException();
            }

            public boolean isWrapperFor(final Class<?> iface) throws SQLException
            {
                throw new UnsupportedOperationException();
            }

            public void setLoginTimeout(final int seconds) throws SQLException
            {
                throw new UnsupportedOperationException();
            }

            public void setLogWriter(final PrintWriter out) throws SQLException
            {
                throw new UnsupportedOperationException();
            }

            public Logger getParentLogger() throws SQLFeatureNotSupportedException
            {
                throw new UnsupportedOperationException();
            }

            public int getLoginTimeout() throws SQLException
            {
                throw new UnsupportedOperationException();
            }

            public PrintWriter getLogWriter() throws SQLException
            {
                throw new UnsupportedOperationException();
            }

            public Connection getConnection(final String username,
                                            final String password) throws SQLException
            {
                throw new UnsupportedOperationException();
            }

            public Connection getConnection() throws SQLException
            {
                return ATeiidTest.this.driver.connect("jdbc:teiid:test", null);
            }
        };
    }

    abstract protected Iterable<ModelMetaData> getModelMetadatas();

    abstract protected Map<String, ExecutionFactory<?, ?>> getExecutionFactories();

    abstract protected Map<String, ConnectionFactoryProvider<?>> getConnectionFactoryProviders();

}
