import java.sql.ResultSet;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Map;

import org.junit.Test;
import org.teiid.adminapi.impl.ModelMetaData;
import org.teiid.jdbc.ConnectionImpl;
import org.teiid.jdbc.RequestOptions;
import org.teiid.jdbc.StatementCallback;
import org.teiid.jdbc.StatementImpl;
import org.teiid.runtime.EmbeddedServer.ConnectionFactoryProvider;
import org.teiid.translator.ExecutionFactory;

import com.ca.chorus.experiement.ContinuousExecutionHandlingDelegateExecutionFactory;
import com.ca.chorus.experiement.MessageBus;
import com.google.common.base.Joiner;
import com.google.common.collect.ImmutableMap;
import com.google.common.collect.Lists;

public class REPL
    extends ATeiidTest
{

    private final MessageBus messageBus = new MessageBus();

    @Test
    public void test() throws Exception
    {
        final ConnectionImpl connection = this.driver.connect("jdbc:teiid:test",
                                                              null);
        final String sql = "SELECT a.c FROM m1.t a ";

        final StatementImpl statement = connection.createStatement();
        statement.submitExecute(sql, new StatementCallback()
        {

            public void onRow(final Statement s, final ResultSet rs) throws Exception
            {
                final Joiner joiner = Joiner.on(" ");
                final ArrayList<String> parts = Lists.newArrayList();
                for (int i = 1; i <= rs.getMetaData().getColumnCount(); i++)
                {
                    parts.add(rs.getString(i));
                }
                // System.out.println(joiner.join(parts));
            }

            public void onException(final Statement s, final Exception e) throws Exception
            {
                e.printStackTrace();
            }

            public void onComplete(final Statement s) throws Exception
            {
                // TODO Auto-generated method stub

            }
        },
                                new RequestOptions().continuous(true));
        Thread.sleep(1000);
        statement.close();
    }

    @Override
    protected Map<String, ConnectionFactoryProvider<?>> getConnectionFactoryProviders()
    {
        return Collections.emptyMap();
    }

    @Override
    protected Map<String, ExecutionFactory<?, ?>> getExecutionFactories()
    {
        final CountingExecutionFactory t1 = new CountingExecutionFactory();
        final CountingExecutionFactory t2 = new CountingExecutionFactory();
        final ContinuousExecutionHandlingDelegateExecutionFactory d1 = new ContinuousExecutionHandlingDelegateExecutionFactory(this.messageBus);
        d1.setDelegate(t1);
        final ContinuousExecutionHandlingDelegateExecutionFactory d2 = new ContinuousExecutionHandlingDelegateExecutionFactory(this.messageBus);
        d2.setDelegate(t2);
        return ImmutableMap.<String, ExecutionFactory<?, ?>> builder()
                           .put("t1", t1)
                           .put("t2", t2)
                           .build();
    }

    @Override
    protected Iterable<ModelMetaData> getModelMetadatas()
    {
        final ModelMetaData m1 = new ModelMetaData();
        m1.setName("m1");
        m1.addSourceMapping("s1", "t1", null);
        final ModelMetaData m2 = new ModelMetaData();
        m2.setName("m2");
        m2.addSourceMapping("s2", "t2", null);
        return Arrays.asList(m1, m2);
    }

    @Override
    protected Map<String, Iterable<String>> getViewsDefinitions()
    {
        return Collections.emptyMap();
    }
}
